@@ -1,7 +1,7 @@
 use std::future::Future;
 
 use ntex::channel::{condition::Condition, condition::Waiter, oneshot};
-use ntex::framed::State as IoState;
+use ntex::io::IoRef;
 use ntex::util::{HashMap, PoolRef, Ready};
 
 use crate::codec::protocol::{self as codec, Begin, Close, End, Error, Frame, Role};
@@ -14,15 +14,15 @@ use crate::{cell::Cell, error::AmqpProtocolError, types::Action, Configuration};
 pub struct Connection(pub(crate) Cell<ConnectionInner>);
 
 pub(crate) struct ConnectionInner {
-    io: IoState,
+    io: IoRef,
     state: ConnectionState,
     codec: AmqpCodec<AmqpFrame>,
     pub(crate) sessions: slab::Slab<SessionState>,
     pub(crate) sessions_map: HashMap<u16, usize>,
     pub(crate) on_close: Condition,
     pub(crate) error: Option<AmqpProtocolError>,
-    channel_max: usize,
-    pub(crate) max_frame_size: usize,
+    channel_max: u16,
+    pub(crate) max_frame_size: u32,
 }
 
 pub(crate) enum SessionState {
@@ -47,7 +47,7 @@ pub(crate) enum ConnectionState {
 
 impl Connection {
     pub(crate) fn new(
-        io: IoState,
+        io: IoRef,
         local_config: &Configuration,
         remote_config: &Configuration,
     ) -> Connection {
@@ -60,7 +60,7 @@ impl Connection {
             error: None,
             on_close: Condition::new(),
             channel_max: local_config.channel_max,
-            max_frame_size: remote_config.max_frame_size as usize,
+            max_frame_size: remote_config.max_frame_size,
         }))
     }
 
@@ -74,7 +74,7 @@ impl Connection {
 
     #[inline]
     /// Check connection state
-    pub fn is_opened(&mut self) -> bool {
+    pub fn is_opened(&self) -> bool {
         let inner = self.0.get_mut();
         if inner.state != ConnectionState::Normal {
             return false;
@@ -133,7 +133,7 @@ impl Connection {
                 let entry = inner.sessions.vacant_entry();
                 let token = entry.key();
 
-                if token >= inner.channel_max {
+                if token >= inner.channel_max as usize {
                     log::trace!("Too many channels: {:?}", token);
                     Err(AmqpProtocolError::TooManyChannels)
                 } else {
@@ -170,8 +170,8 @@ impl Connection {
         log::trace!("outgoing: {:#?}", frame);
 
         let inner = self.0.get_mut();
-        if let Err(e) = inner.io.write().encode(frame, &inner.codec) {
-            inner.set_error(e.into());
+        if let Err(e) = inner.io.encode(frame, &inner.codec) {
+            inner.set_error(e.into())
         }
     }
 
@@ -211,8 +211,8 @@ impl ConnectionInner {
         #[cfg(feature = "frame-trace")]
         log::trace!("outgoing: {:#?}", frame);
 
-        if let Err(e) = self.io.write().encode(frame, &self.codec) {
-            self.set_error(e.into());
+        if let Err(e) = self.io.encode(frame, &self.codec) {
+            self.set_error(e.into())
         }
     }
 
@@ -251,7 +251,6 @@ impl ConnectionInner {
         }));
 
         self.io
-            .write()
             .encode(AmqpFrame::new(token as u16, begin.into()), &self.codec)
             .map(|_| ())
             .map_err(AmqpProtocolError::Codec)
@@ -316,7 +315,7 @@ impl ConnectionInner {
                     return Ok(Action::None);
                 } else {
                     log::trace!("Connection closed remotely: {:?}", close);
-                    let err = AmqpProtocolError::Closed(close.error.clone());
+                    let err = AmqpProtocolError::Closed(close.error);
                     self.set_error(err.clone());
                     let close = Close { error: None };
                     self.post_frame(AmqpFrame::new(0, close.into()));
@@ -378,8 +377,8 @@ impl ConnectionInner {
                                 // receiver link
                                 let link = session
                                     .get_mut()
-                                    .attach_remote_receiver_link(session.clone(), attach);
-                                Ok(Action::AttachReceiver(link))
+                                    .attach_remote_receiver_link(session.clone(), &attach);
+                                Ok(Action::AttachReceiver(link, attach))
                             }
                         }
                     }
