@@ -1,4 +1,4 @@
-use std::fmt;
+use std::{fmt, io};
 
 use ntex::util::Either;
 use ntex_amqp_codec::protocol;
@@ -26,13 +26,14 @@ impl fmt::Debug for ControlFrame {
 
 #[derive(Debug)]
 pub enum ControlFrameKind {
-    AttachReceiver(ReceiverLink),
+    AttachReceiver(protocol::Attach, ReceiverLink),
     AttachSender(protocol::Attach, SenderLink),
     Flow(protocol::Flow, SenderLink),
     DetachSender(protocol::Detach, SenderLink),
     DetachReceiver(protocol::Detach, ReceiverLink),
     SessionEnded(Vec<Either<SenderLink, ReceiverLink>>),
     ProtocolError(AmqpProtocolError),
+    Disconnected(Option<io::Error>),
     Closed(bool),
 }
 
@@ -59,12 +60,6 @@ impl ControlFrame {
         self.0.get_ref().session.as_ref().unwrap()
     }
 
-    #[inline]
-    #[doc(hidden)]
-    pub fn frame(&self) -> &ControlFrameKind {
-        &self.0.kind
-    }
-
     #[inline]
     pub fn kind(&self) -> &ControlFrameKind {
         &self.0.kind
