@@ -1,17 +1,17 @@
-use std::{cell, fmt, future::Future, marker, pin::Pin, rc::Rc, task::Context, task::Poll};
+use std::{cell, future::Future, marker, pin::Pin, task::Context, task::Poll};
 
-use ntex::framed::DispatchItem;
+use ntex::io::DispatchItem;
 use ntex::service::Service;
 use ntex::time::{sleep, Millis, Sleep};
-use ntex::util::{Either, Ready};
+use ntex::util::{ready, Either, Ready};
 
 use crate::codec::protocol::Frame;
 use crate::codec::{AmqpCodec, AmqpFrame};
-use crate::error::{AmqpProtocolError, DispatcherError, Error};
+use crate::error::{AmqpDispatcherError, AmqpProtocolError, Error};
 use crate::{connection::Connection, types, ControlFrame, ControlFrameKind, ReceiverLink};
 
 /// Amqp server dispatcher service.
-pub(crate) struct Dispatcher<Sr: Service, Ctl: Service> {
+pub(crate) struct Dispatcher<Sr, Ctl: Service<ControlFrame>> {
     sink: Connection,
     service: Sr,
     ctl_service: Ctl,
@@ -23,11 +23,9 @@ pub(crate) struct Dispatcher<Sr: Service, Ctl: Service> {
 
 impl<Sr, Ctl> Dispatcher<Sr, Ctl>
 where
-    Sr: Service<Request = types::Message, Response = ()>,
-    Sr::Error: Into<Error> + 'static,
-    Sr::Future: 'static,
-    Ctl: Service<Request = ControlFrame, Response = ()>,
-    Ctl::Error: Into<Error> + 'static,
+    Sr: Service<types::Message, Response = ()> + 'static,
+    Ctl: Service<ControlFrame, Response = ()> + 'static,
+    Error: From<Sr::Error> + From<Ctl::Error>,
 {
     pub(crate) fn new(
         sink: Connection,
@@ -58,7 +56,7 @@ where
         }
     }
 
-    fn handle_control_fut(&self, cx: &mut Context<'_>) -> Result<bool, DispatcherError> {
+    fn handle_control_fut(&self, cx: &mut Context<'_>) -> Result<bool, AmqpDispatcherError> {
         let mut inner = self.ctl_fut.borrow_mut();
 
         // process control frame
@@ -86,10 +84,10 @@ where
         &self,
         frame: &ControlFrame,
         err: Option<Error>,
-    ) -> Result<(), DispatcherError> {
+    ) -> Result<(), AmqpDispatcherError> {
         if let Some(err) = err {
             match &frame.0.get_mut().kind {
-                ControlFrameKind::AttachReceiver(ref link) => {
+                ControlFrameKind::AttachReceiver(_, ref link) => {
                     let _ = link.close_with_error(err);
                 }
                 ControlFrameKind::AttachSender(ref frm, _) => {
@@ -111,21 +109,24 @@ where
                     self.sink.set_error(err.clone());
                     return Err(err.clone().into());
                 }
-                ControlFrameKind::Closed(_) => {
+                ControlFrameKind::Closed(_) | ControlFrameKind::Disconnected(_) => {
                     self.sink.set_error(AmqpProtocolError::Disconnected);
                 }
                 ControlFrameKind::SessionEnded(_) => (),
             }
         } else {
             match frame.0.get_mut().kind {
-                ControlFrameKind::AttachReceiver(ref link) => {
+                ControlFrameKind::AttachReceiver(ref frm, ref link) => {
                     let link = link.clone();
-                    let fut = self.service.call(types::Message::Attached(link.clone()));
+                    let frm = frm.clone();
+                    let fut = self
+                        .service
+                        .call(types::Message::Attached(frm.clone(), link.clone()));
                     ntex::rt::spawn(async move {
                         if let Err(err) = fut.await {
-                            let _ = link.close_with_error(err.into()).await;
+                            let _ = link.close_with_error(Error::from(err)).await;
                         } else {
-                            link.confirm_receiver_link();
+                            link.confirm_receiver_link(&frm);
                             link.set_link_credit(50);
                         }
                     });
@@ -149,7 +150,7 @@ where
                     self.sink.set_error(err.clone());
                     return Err(err.clone().into());
                 }
-                ControlFrameKind::Closed(_) => {
+                ControlFrameKind::Closed(_) | ControlFrameKind::Disconnected(_) => {
                     self.sink.set_error(AmqpProtocolError::Disconnected);
                 }
                 ControlFrameKind::SessionEnded(_) => (),
@@ -159,18 +160,14 @@ where
     }
 }
 
-impl<Sr, Ctl> Service for Dispatcher<Sr, Ctl>
+impl<Sr, Ctl> Service<DispatchItem<AmqpCodec<AmqpFrame>>> for Dispatcher<Sr, Ctl>
 where
-    Sr: Service<Request = types::Message, Response = ()>,
-    Sr::Error: Into<Error> + fmt::Debug + 'static,
-    Sr::Future: 'static,
-    Ctl: Service<Request = ControlFrame, Response = ()>,
-    Ctl::Error: Into<Error> + fmt::Debug + 'static,
-    Ctl::Future: 'static,
+    Sr: Service<types::Message, Response = ()> + 'static,
+    Ctl: Service<ControlFrame, Response = ()> + 'static,
+    Error: From<Sr::Error> + From<Ctl::Error>,
 {
-    type Request = DispatchItem<AmqpCodec<AmqpFrame>>;
-    type Response = ();
-    type Error = DispatcherError;
+    type Response = Option<AmqpFrame>;
+    type Error = AmqpDispatcherError;
     type Future = Either<ServiceResult<Sr::Future, Sr::Error>, Ready<Self::Response, Self::Error>>;
 
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
@@ -182,14 +179,16 @@ where
 
         // check readiness
         let res1 = self.service.poll_ready(cx).map_err(|err| {
+            let err = Error::from(err);
             error!("Publish service readiness check failed: {:?}", err);
-            let _ = self.sink.close_with_error(err.into());
-            DispatcherError::Service
+            let _ = self.sink.close_with_error(err);
+            AmqpDispatcherError::Service
         })?;
         let res2 = self.ctl_service.poll_ready(cx).map_err(|err| {
+            let err = Error::from(err);
             error!("Control service readiness check failed: {:?}", err);
-            let _ = self.sink.close_with_error(err.into());
-            DispatcherError::Service
+            let _ = self.sink.close_with_error(err);
+            AmqpDispatcherError::Service
         })?;
 
         if res0 || res1.is_pending() || res2.is_pending() {
@@ -221,7 +220,7 @@ where
         }
     }
 
-    fn call(&self, request: Self::Request) -> Self::Future {
+    fn call(&self, request: DispatchItem<AmqpCodec<AmqpFrame>>) -> Self::Future {
         match request {
             DispatchItem::Item(frame) => {
                 #[cfg(feature = "frame-trace")]
@@ -230,7 +229,7 @@ where
                 let action = match self
                     .sink
                     .handle_frame(frame)
-                    .map_err(DispatcherError::Protocol)
+                    .map_err(AmqpDispatcherError::Protocol)
                 {
                     Ok(a) => a,
                     Err(e) => return Either::Right(Ready::Err(e)),
@@ -261,10 +260,10 @@ where
                         *self.ctl_fut.borrow_mut() =
                             Some((Some(frame.clone()), Box::pin(self.ctl_service.call(frame))));
                     }
-                    types::Action::AttachReceiver(link) => {
+                    types::Action::AttachReceiver(link, frm) => {
                         let frame = ControlFrame::new(
                             link.session().inner.clone(),
-                            ControlFrameKind::AttachReceiver(link),
+                            ControlFrameKind::AttachReceiver(frm, link),
                         );
                         *self.ctl_fut.borrow_mut() =
                             Some((Some(frame.clone()), Box::pin(self.ctl_service.call(frame))));
@@ -298,13 +297,13 @@ where
                     types::Action::None => (),
                 };
 
-                Either::Right(Ready::Ok(()))
+                Either::Right(Ready::Ok(None))
             }
             DispatchItem::EncoderError(err) | DispatchItem::DecoderError(err) => {
                 let frame = ControlFrame::new_kind(ControlFrameKind::ProtocolError(err.into()));
                 *self.ctl_fut.borrow_mut() =
                     Some((Some(frame.clone()), Box::pin(self.ctl_service.call(frame))));
-                Either::Right(Ready::Ok(()))
+                Either::Right(Ready::Ok(None))
             }
             DispatchItem::KeepAliveTimeout => {
                 let frame = ControlFrame::new_kind(ControlFrameKind::ProtocolError(
@@ -312,18 +311,16 @@ where
                 ));
                 *self.ctl_fut.borrow_mut() =
                     Some((Some(frame.clone()), Box::pin(self.ctl_service.call(frame))));
-                Either::Right(Ready::Ok(()))
+                Either::Right(Ready::Ok(None))
             }
-            DispatchItem::IoError(err) => {
-                let frame = ControlFrame::new_kind(ControlFrameKind::ProtocolError(
-                    AmqpProtocolError::Io(Rc::new(err)),
-                ));
+            DispatchItem::Disconnect(e) => {
+                let frame = ControlFrame::new_kind(ControlFrameKind::Disconnected(e));
                 *self.ctl_fut.borrow_mut() =
                     Some((Some(frame.clone()), Box::pin(self.ctl_service.call(frame))));
-                Either::Right(Ready::Ok(()))
+                Either::Right(Ready::Ok(None))
             }
             DispatchItem::WBackPressureEnabled | DispatchItem::WBackPressureDisabled => {
-                Either::Right(Ready::Ok(()))
+                Either::Right(Ready::Ok(None))
             }
         }
     }
@@ -341,21 +338,20 @@ pin_project_lite::pin_project! {
 impl<F, E> Future for ServiceResult<F, E>
 where
     F: Future<Output = Result<(), E>>,
-    E: Into<Error> + fmt::Debug,
+    E: Into<Error>,
 {
-    type Output = Result<(), DispatcherError>;
+    type Output = Result<Option<AmqpFrame>, AmqpDispatcherError>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         let this = self.project();
 
-        match this.fut.poll(cx) {
-            Poll::Pending => Poll::Pending,
-            Poll::Ready(Ok(_)) => Poll::Ready(Ok(())),
-            Poll::Ready(Err(e)) => {
-                log::trace!("Service error {:?}", e);
-                let _ = this.link.close_with_error(e.into());
-                Poll::Ready(Ok::<_, DispatcherError>(()))
-            }
+        if let Err(e) = ready!(this.fut.poll(cx)) {
+            let e = e.into();
+            log::trace!("Service error {:?}", e);
+            let _ = this.link.close_with_error(e);
+            Poll::Ready(Ok::<_, AmqpDispatcherError>(None))
+        } else {
+            Poll::Ready(Ok(None))
         }
     }
 }
