@@ -8,43 +8,29 @@ use crate::{codec::protocol, types::Outcome};
 
 /// Errors which can occur when attempting to handle amqp connection.
 #[derive(Debug, Display, From)]
-pub enum DispatcherError {
+pub enum AmqpDispatcherError {
     #[display(fmt = "Service error")]
     /// Service error
     Service,
-    /// Amqp codec error
-    #[display(fmt = "Amqp codec error: {:?}", _0)]
-    Codec(AmqpCodecError),
     /// Amqp protocol error
     #[display(fmt = "Amqp protocol error: {:?}", _0)]
     Protocol(AmqpProtocolError),
     /// Peer disconnect
-    Disconnected,
-    /// Unexpected io error
-    Io(io::Error),
+    #[display(fmt = "Peer disconnected error: {:?}", _0)]
+    Disconnected(Option<io::Error>),
 }
 
-impl error::Error for DispatcherError {}
+impl error::Error for AmqpDispatcherError {}
 
-impl Clone for DispatcherError {
+impl Clone for AmqpDispatcherError {
     fn clone(&self) -> Self {
         match self {
-            DispatcherError::Service => DispatcherError::Service,
-            DispatcherError::Codec(err) => DispatcherError::Codec(err.clone()),
-            DispatcherError::Protocol(err) => DispatcherError::Protocol(err.clone()),
-            DispatcherError::Disconnected => DispatcherError::Disconnected,
-            DispatcherError::Io(err) => {
-                DispatcherError::Io(io::Error::new(err.kind(), format!("{}", err)))
-            }
-        }
-    }
-}
-
-impl From<Either<AmqpCodecError, io::Error>> for DispatcherError {
-    fn from(err: Either<AmqpCodecError, io::Error>) -> Self {
-        match err {
-            Either::Left(err) => DispatcherError::Codec(err),
-            Either::Right(err) => DispatcherError::Io(err),
+            AmqpDispatcherError::Service => AmqpDispatcherError::Service,
+            AmqpDispatcherError::Protocol(err) => AmqpDispatcherError::Protocol(err.clone()),
+            AmqpDispatcherError::Disconnected(Some(ref err)) => AmqpDispatcherError::Disconnected(
+                Some(io::Error::new(err.kind(), format!("{}", err))),
+            ),
+            AmqpDispatcherError::Disconnected(None) => AmqpDispatcherError::Disconnected(None),
         }
     }
 }
@@ -56,7 +42,6 @@ pub enum AmqpProtocolError {
     BodyTooLarge,
     KeepAliveTimeout,
     Disconnected,
-    Io,
     #[display(fmt = "Unknown session: {:?}", _0)]
     UnknownSession(protocol::Frame),
     #[display(fmt = "Unknown link in session: {:?}", _0)]
