@@ -1,7 +1,7 @@
 use std::{collections::VecDeque, future::Future, hash, pin::Pin, task::Context, task::Poll};
 
-use ntex::util::{ByteString, BytesMut};
-use ntex::{channel::oneshot, task::LocalWaker, Stream};
+use ntex::util::{poll_fn, ByteString, BytesMut, PoolRef, Stream};
+use ntex::{channel::oneshot, task::LocalWaker};
 use ntex_amqp_codec::protocol::{
     self as codec, Attach, DeliveryNumber, Disposition, Error, Handle, LinkError,
     ReceiverSettleMode, Role, SenderSettleMode, Source, TerminusDurability, TerminusExpiryPolicy,
@@ -37,6 +37,10 @@ impl ReceiverLink {
         ReceiverLink { inner }
     }
 
+    pub fn name(&self) -> &ByteString {
+        &self.inner.get_ref().name
+    }
+
     pub fn handle(&self) -> Handle {
         self.inner.get_ref().handle as Handle
     }
@@ -53,14 +57,6 @@ impl ReceiverLink {
         &self.inner.get_ref().session
     }
 
-    pub fn session_mut(&mut self) -> &mut Session {
-        &mut self.inner.get_mut().session
-    }
-
-    pub fn frame(&self) -> &Attach {
-        &self.inner.get_ref().attach
-    }
-
     pub fn is_closed(&self) -> bool {
         self.inner.get_ref().closed
     }
@@ -69,19 +65,26 @@ impl ReceiverLink {
         self.inner.get_ref().error.as_ref()
     }
 
-    pub(crate) fn confirm_receiver_link(&self) {
+    pub(crate) fn confirm_receiver_link(&self, frm: &Attach) {
         let inner = self.inner.get_mut();
+        let size = self.inner.get_ref().max_message_size;
+        let size = if size != 0 { Some(size) } else { None };
         inner
             .session
             .inner
             .get_mut()
-            .confirm_receiver_link(inner.handle, &inner.attach);
+            .confirm_receiver_link(inner.handle, frm, size);
     }
 
     pub fn set_link_credit(&self, credit: u32) {
         self.inner.get_mut().set_link_credit(credit);
     }
 
+    /// Set max message size.
+    pub fn set_max_message_size(&self, size: u64) {
+        self.inner.get_mut().max_message_size = size;
+    }
+
     /// Set max total size for partial transfers.
     ///
     /// Default is 256Kb
@@ -132,18 +135,31 @@ impl ReceiverLink {
     }
 
     pub(crate) fn remote_closed(&self, error: Option<Error>) {
-        trace!("Receiver link has been closed remotely");
         let inner = self.inner.get_mut();
+        trace!(
+            "Receiver link has been closed remotely handle: {:?} name: {:?}",
+            inner.remote_handle,
+            inner.name
+        );
         inner.closed = true;
         inner.error = error;
         inner.wake();
     }
-}
 
-impl Stream for ReceiverLink {
-    type Item = Result<Transfer, AmqpProtocolError>;
+    /// Attempt to pull out the next value of this receiver, registering
+    /// the current task for wakeup if the value is not yet available,
+    /// and returning None if the stream is exhausted.
+    pub async fn recv(&self) -> Option<Result<Transfer, AmqpProtocolError>> {
+        poll_fn(|cx| self.poll_recv(cx)).await
+    }
 
-    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+    /// Attempt to pull out the next value of this receiver, registering
+    /// the current task for wakeup if the value is not yet available,
+    /// and returning None if the stream is exhausted.
+    pub fn poll_recv(
+        &self,
+        cx: &mut Context<'_>,
+    ) -> Poll<Option<Result<Transfer, AmqpProtocolError>>> {
         let inner = self.inner.get_mut();
 
         if inner.partial_body.is_some() && inner.queue.len() == 1 {
@@ -172,11 +188,19 @@ impl Stream for ReceiverLink {
     }
 }
 
+impl Stream for ReceiverLink {
+    type Item = Result<Transfer, AmqpProtocolError>;
+
+    fn poll_next(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Option<Self::Item>> {
+        self.poll_recv(cx)
+    }
+}
+
 #[derive(Debug)]
 pub(crate) struct ReceiverLinkInner {
+    name: ByteString,
     handle: Handle,
     remote_handle: Handle,
-    attach: Attach,
     session: Session,
     closed: bool,
     reader_task: LocalWaker,
@@ -186,6 +210,8 @@ pub(crate) struct ReceiverLinkInner {
     error: Option<Error>,
     partial_body: Option<BytesMut>,
     partial_body_max: usize,
+    max_message_size: u64,
+    pool: PoolRef,
 }
 
 impl ReceiverLinkInner {
@@ -193,11 +219,17 @@ impl ReceiverLinkInner {
         session: Cell<SessionInner>,
         handle: Handle,
         remote_handle: Handle,
-        attach: Attach,
+        frame: &Attach,
     ) -> ReceiverLinkInner {
+        let pool = session.get_ref().memory_pool();
+        let mut name = frame.name().clone();
+        name.trimdown();
+
         ReceiverLinkInner {
+            name,
             handle,
             remote_handle,
+            pool,
             session: Session::new(session),
             closed: false,
             queue: VecDeque::with_capacity(4),
@@ -205,9 +237,9 @@ impl ReceiverLinkInner {
             error: None,
             partial_body: None,
             partial_body_max: 262_144,
-            delivery_count: attach.initial_delivery_count().unwrap_or(0),
+            delivery_count: frame.initial_delivery_count().unwrap_or(0),
+            max_message_size: frame.max_message_size().unwrap_or(0),
             reader_task: LocalWaker::new(),
-            attach,
         }
     }
 
@@ -346,15 +378,15 @@ impl ReceiverLinkInner {
                 } else {
                     let body = if let Some(body) = transfer.0.body.take() {
                         match body {
-                            TransferBody::Data(data) => BytesMut::from(data.as_ref()),
+                            TransferBody::Data(data) => BytesMut::copy_from_slice(&data),
                             TransferBody::Message(msg) => {
-                                let mut buf = BytesMut::with_capacity(msg.encoded_size());
+                                let mut buf = self.pool.buf_with_capacity(msg.encoded_size());
                                 msg.encode(&mut buf);
                                 buf
                             }
                         }
                     } else {
-                        BytesMut::new()
+                        self.pool.buf_with_capacity(16)
                     };
                     self.partial_body = Some(body);
                     self.queue.push_back(transfer);
@@ -451,9 +483,4 @@ impl ReceiverLinkBuilder {
             Err(_) => Err(AmqpProtocolError::Disconnected),
         }
     }
-
-    #[doc(hidden)]
-    pub async fn open(self) -> Result<ReceiverLink, AmqpProtocolError> {
-        self.attach().await
-    }
 }
