@@ -1,7 +1,6 @@
 use std::{fmt, rc::Rc};
 
-use ntex::codec::{AsyncRead, AsyncWrite};
-use ntex::framed::State;
+use ntex::io::IoBoxed;
 use ntex::util::{ByteString, Bytes};
 
 use crate::codec::protocol::{
@@ -12,14 +11,13 @@ use crate::codec::{AmqpCodec, AmqpFrame, ProtocolIdCodec, ProtocolIdError, SaslF
 use super::{handshake::HandshakeAmqpOpened, HandshakeError};
 use crate::{connection::Connection, Configuration};
 
-pub struct Sasl<Io> {
-    io: Io,
-    state: State,
+pub struct Sasl {
+    state: IoBoxed,
     mechanisms: Symbols,
     local_config: Rc<Configuration>,
 }
 
-impl<Io> fmt::Debug for Sasl<Io> {
+impl fmt::Debug for Sasl {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt.debug_struct("SaslAuth")
             .field("mechanisms", &self.mechanisms)
@@ -27,10 +25,9 @@ impl<Io> fmt::Debug for Sasl<Io> {
     }
 }
 
-impl<Io> Sasl<Io> {
-    pub(crate) fn new(io: Io, state: State, local_config: Rc<Configuration>) -> Self {
+impl Sasl {
+    pub(crate) fn new(state: IoBoxed, local_config: Rc<Configuration>) -> Self {
         Sasl {
-            io,
             state,
             local_config,
             mechanisms: Symbols::default(),
@@ -38,18 +35,10 @@ impl<Io> Sasl<Io> {
     }
 }
 
-impl<Io> Sasl<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin,
-{
+impl Sasl {
     /// Returns reference to io object
-    pub fn get_ref(&self) -> &Io {
-        &self.io
-    }
-
-    /// Returns mutable reference to io object
-    pub fn get_mut(&mut self) -> &mut Io {
-        &mut self.io
+    pub fn io(&self) -> &IoBoxed {
+        &self.state
     }
 
     /// Add supported sasl mechanism
@@ -59,9 +48,8 @@ where
     }
 
     /// Initialize sasl auth procedure
-    pub async fn init(self) -> Result<SaslInit<Io>, HandshakeError> {
+    pub async fn init(self) -> Result<SaslInit, HandshakeError> {
         let Sasl {
-            mut io,
             state,
             mechanisms,
             local_config,
@@ -75,19 +63,17 @@ where
 
         let codec = AmqpCodec::<SaslFrame>::new();
         state
-            .send(&mut io, &codec, frame)
+            .send(frame, &codec)
             .await
             .map_err(HandshakeError::from)?;
         let frame = state
-            .next(&mut io, &codec)
-            .await
-            .map_err(HandshakeError::from)?
-            .ok_or(HandshakeError::Disconnected)?;
+            .recv(&codec)
+            .await?
+            .ok_or(HandshakeError::Disconnected(None))?;
 
         match frame.body {
             SaslFrameBody::SaslInit(frame) => Ok(SaslInit {
                 frame,
-                io,
                 state,
                 codec,
                 local_config,
@@ -98,15 +84,14 @@ where
 }
 
 /// Initialization stage of sasl negotiation
-pub struct SaslInit<Io> {
+pub struct SaslInit {
     frame: protocol::SaslInit,
-    io: Io,
-    state: State,
+    state: IoBoxed,
     codec: AmqpCodec<SaslFrame>,
     local_config: Rc<Configuration>,
 }
 
-impl<Io> fmt::Debug for SaslInit<Io> {
+impl fmt::Debug for SaslInit {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt.debug_struct("SaslInit")
             .field("frame", &self.frame)
@@ -114,10 +99,12 @@ impl<Io> fmt::Debug for SaslInit<Io> {
     }
 }
 
-impl<Io> SaslInit<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin,
-{
+impl SaslInit {
+    /// Returns reference to io object
+    pub fn io(&self) -> &IoBoxed {
+        &self.state
+    }
+
     /// Sasl mechanism
     pub fn mechanism(&self) -> &str {
         self.frame.mechanism.as_str()
@@ -133,46 +120,30 @@ where
         self.frame.hostname.as_ref().map(AsRef::as_ref)
     }
 
-    /// Returns reference to io object
-    pub fn get_ref(&self) -> &Io {
-        &self.io
-    }
-
-    /// Returns mutable reference to io object
-    pub fn get_mut(&mut self) -> &mut Io {
-        &mut self.io
-    }
-
     /// Initiate sasl challenge
-    pub async fn challenge(self) -> Result<SaslResponse<Io>, HandshakeError> {
+    pub async fn challenge(self) -> Result<SaslResponse, HandshakeError> {
         self.challenge_with(Bytes::new()).await
     }
 
     /// Initiate sasl challenge with challenge payload
-    pub async fn challenge_with(
-        self,
-        challenge: Bytes,
-    ) -> Result<SaslResponse<Io>, HandshakeError> {
-        let mut io = self.io;
+    pub async fn challenge_with(self, challenge: Bytes) -> Result<SaslResponse, HandshakeError> {
         let state = self.state;
         let codec = self.codec;
         let local_config = self.local_config;
         let frame = SaslChallenge { challenge }.into();
 
         state
-            .send(&mut io, &codec, frame)
+            .send(frame, &codec)
             .await
             .map_err(HandshakeError::from)?;
         let frame = state
-            .next(&mut io, &codec)
-            .await
-            .map_err(HandshakeError::from)?
-            .ok_or(HandshakeError::Disconnected)?;
+            .recv(&codec)
+            .await?
+            .ok_or(HandshakeError::Disconnected(None))?;
 
         match frame.body {
             SaslFrameBody::SaslResponse(frame) => Ok(SaslResponse {
                 frame,
-                io,
                 state,
                 codec,
                 local_config,
@@ -182,8 +153,7 @@ where
     }
 
     /// Sasl challenge outcome
-    pub async fn outcome(self, code: SaslCode) -> Result<SaslSuccess<Io>, HandshakeError> {
-        let mut io = self.io;
+    pub async fn outcome(self, code: SaslCode) -> Result<SaslSuccess, HandshakeError> {
         let state = self.state;
         let codec = self.codec;
         let local_config = self.local_config;
@@ -194,27 +164,25 @@ where
         }
         .into();
         state
-            .send(&mut io, &codec, frame)
+            .send(frame, &codec)
             .await
             .map_err(HandshakeError::from)?;
 
         Ok(SaslSuccess {
-            io,
             state,
             local_config,
         })
     }
 }
 
-pub struct SaslResponse<Io> {
+pub struct SaslResponse {
     frame: protocol::SaslResponse,
-    io: Io,
-    state: State,
+    state: IoBoxed,
     codec: AmqpCodec<SaslFrame>,
     local_config: Rc<Configuration>,
 }
 
-impl<Io> fmt::Debug for SaslResponse<Io> {
+impl fmt::Debug for SaslResponse {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt.debug_struct("SaslResponse")
             .field("frame", &self.frame)
@@ -222,18 +190,19 @@ impl<Io> fmt::Debug for SaslResponse<Io> {
     }
 }
 
-impl<Io> SaslResponse<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin,
-{
+impl SaslResponse {
+    /// Returns reference to io object
+    pub fn io(&self) -> &IoBoxed {
+        &self.state
+    }
+
     /// Client response payload
     pub fn response(&self) -> &[u8] {
         &self.frame.response[..]
     }
 
     /// Sasl challenge outcome
-    pub async fn outcome(self, code: SaslCode) -> Result<SaslSuccess<Io>, HandshakeError> {
-        let mut io = self.io;
+    pub async fn outcome(self, code: SaslCode) -> Result<SaslSuccess, HandshakeError> {
         let state = self.state;
         let codec = self.codec;
         let local_config = self.local_config;
@@ -244,69 +213,55 @@ where
         }
         .into();
         state
-            .send(&mut io, &codec, frame)
+            .send(frame, &codec)
             .await
             .map_err(HandshakeError::from)?;
         state
-            .next(&mut io, &codec)
-            .await
-            .map_err(HandshakeError::from)?
-            .ok_or(HandshakeError::Disconnected)?;
+            .recv(&codec)
+            .await?
+            .ok_or(HandshakeError::Disconnected(None))?;
 
         Ok(SaslSuccess {
-            io,
             state,
             local_config,
         })
     }
 }
 
-pub struct SaslSuccess<Io> {
-    io: Io,
-    state: State,
+pub struct SaslSuccess {
+    state: IoBoxed,
     local_config: Rc<Configuration>,
 }
 
-impl<Io> SaslSuccess<Io>
-where
-    Io: AsyncRead + AsyncWrite + Unpin,
-{
+impl SaslSuccess {
     /// Returns reference to io object
-    pub fn get_ref(&self) -> &Io {
-        &self.io
-    }
-
-    /// Returns mutable reference to io object
-    pub fn get_mut(&mut self) -> &mut Io {
-        &mut self.io
+    pub fn io(&self) -> &IoBoxed {
+        &self.state
     }
 
     /// Wait for connection open frame
-    pub async fn open(self) -> Result<HandshakeAmqpOpened<Io>, HandshakeError> {
-        let mut io = self.io;
+    pub async fn open(self) -> Result<HandshakeAmqpOpened, HandshakeError> {
         let state = self.state;
 
         let protocol = state
-            .next(&mut io, &ProtocolIdCodec)
-            .await
-            .map_err(HandshakeError::from)?
-            .ok_or(HandshakeError::Disconnected)?;
+            .recv(&ProtocolIdCodec)
+            .await?
+            .ok_or(HandshakeError::Disconnected(None))?;
 
         match protocol {
             ProtocolId::Amqp => {
                 // confirm protocol
                 state
-                    .send(&mut io, &ProtocolIdCodec, ProtocolId::Amqp)
+                    .send(ProtocolId::Amqp, &ProtocolIdCodec)
                     .await
                     .map_err(HandshakeError::from)?;
 
                 // Wait for connection open frame
                 let codec = AmqpCodec::<AmqpFrame>::new();
                 let frame = state
-                    .next(&mut io, &codec)
-                    .await
-                    .map_err(HandshakeError::from)?
-                    .ok_or(HandshakeError::Disconnected)?;
+                    .recv(&codec)
+                    .await?
+                    .ok_or(HandshakeError::Disconnected(None))?;
 
                 let frame = frame.into_parts().1;
                 match frame {
@@ -319,7 +274,6 @@ where
 
                         Ok(HandshakeAmqpOpened::new(
                             frame,
-                            io,
                             sink,
                             state,
                             local_config,
