@@ -1,7 +1,7 @@
-use std::{collections::VecDeque, fmt, future::ready, future::Future};
+use std::{collections::VecDeque, fmt, future::Future};
 
 use ntex::channel::{condition, oneshot, pool};
-use ntex::util::{ByteString, Bytes, Either, HashMap, PoolRef};
+use ntex::util::{ByteString, Bytes, Either, HashMap, PoolRef, Ready};
 use slab::Slab;
 
 use ntex_amqp_codec::protocol::{
@@ -39,17 +39,31 @@ impl Session {
         &self.inner.get_ref().sink
     }
 
-    pub fn end(&self) -> impl Future<Output = ()> {
+    /// Get remote window size
+    pub fn remote_window_size(&self) -> u32 {
+        self.inner.get_ref().remote_incoming_window
+    }
+
+    pub fn end(&self) -> impl Future<Output = Result<(), AmqpProtocolError>> {
         let inner = self.inner.get_mut();
 
         if inner.flags.contains(Flags::ENDED) {
-            Either::Left(ready(()))
-        } else if inner.flags.contains(Flags::ENDING) {
-            Either::Right(inner.closed.wait())
+            Either::Left(Ready::Ok(()))
         } else {
-            inner.post_frame(Frame::End(End { error: None }));
-            inner.flags.insert(Flags::ENDING);
-            Either::Right(inner.closed.wait())
+            if !inner.flags.contains(Flags::ENDING) {
+                inner.sink.close_session(inner.remote_channel_id as usize);
+                inner.post_frame(Frame::End(End { error: None }));
+                inner.flags.insert(Flags::ENDING);
+            }
+            let inner = self.inner.clone();
+            Either::Right(async move {
+                inner.closed.wait().await;
+                if let Some(err @ AmqpProtocolError::SessionEnded(Some(_))) = inner.error.clone() {
+                    Err(err)
+                } else {
+                    Ok(())
+                }
+            })
         }
     }
 
@@ -86,7 +100,7 @@ impl Session {
 
     /// Open receiver link
     pub fn build_receiver_link<T: Into<ByteString>, U: Into<ByteString>>(
-        &mut self,
+        &self,
         name: U,
         address: T,
     ) -> ReceiverLinkBuilder {
@@ -97,7 +111,7 @@ impl Session {
 
     /// Detach receiver link
     pub fn detach_receiver_link(
-        &mut self,
+        &self,
         handle: Handle,
         error: Option<Error>,
     ) -> impl Future<Output = Result<(), AmqpProtocolError>> {
@@ -124,7 +138,7 @@ impl Session {
 
     /// Detach sender link
     pub fn detach_sender_link(
-        &mut self,
+        &self,
         handle: Handle,
         error: Option<Error>,
     ) -> impl Future<Output = Result<(), AmqpProtocolError>> {
@@ -150,7 +164,7 @@ impl Session {
     }
 
     pub fn wait_disposition(
-        &mut self,
+        &self,
         id: DeliveryNumber,
     ) -> impl Future<Output = Result<Disposition, AmqpProtocolError>> {
         self.inner.get_mut().wait_disposition(id)
@@ -359,7 +373,7 @@ impl SessionInner {
         async move { rx.await.map_err(|_| AmqpProtocolError::Disconnected) }
     }
 
-    pub(crate) fn max_frame_size(&self) -> usize {
+    pub(crate) fn max_frame_size(&self) -> u32 {
         self.sink.0.max_frame_size
     }
 
@@ -500,7 +514,7 @@ impl SessionInner {
     pub(crate) fn attach_remote_receiver_link(
         &mut self,
         cell: Cell<SessionInner>,
-        attach: Attach,
+        attach: &Attach,
     ) -> ReceiverLink {
         let handle = attach.handle();
         let entry = self.links.vacant_entry();
@@ -528,7 +542,7 @@ impl SessionInner {
             cell,
             token as u32,
             token as u32,
-            frame.clone(),
+            &frame,
         ));
         entry.insert(Either::Right(ReceiverLinkState::OpeningLocal(Some((
             inner, tx,
@@ -541,11 +555,17 @@ impl SessionInner {
         rx
     }
 
-    pub(crate) fn confirm_receiver_link(&mut self, token: Handle, attach: &Attach) {
+    pub(crate) fn confirm_receiver_link(
+        &mut self,
+        token: Handle,
+        attach: &Attach,
+        max_message_size: Option<u64>,
+    ) {
         if let Some(Either::Right(link)) = self.links.get_mut(token as usize) {
             if let ReceiverLinkState::Opening(l) = link {
                 if let Some(l) = l.take() {
                     let attach = Attach(Box::new(codec::AttachInner {
+                        max_message_size,
                         name: attach.0.name.clone(),
                         handle: token as Handle,
                         role: Role::Receiver,
@@ -556,7 +576,6 @@ impl SessionInner {
                         unsettled: None,
                         incomplete_unsettled: false,
                         initial_delivery_count: Some(0),
-                        max_message_size: attach.0.max_message_size,
                         offered_capabilities: None,
                         desired_capabilities: None,
                         properties: None,
