@@ -2,18 +2,15 @@ use std::collections::VecDeque;
 use std::{convert::TryFrom, future::Future, mem, pin::Pin, task::Context, task::Poll};
 
 use ntex::channel::{condition, oneshot, pool};
-use ntex::util::{BufMut, ByteString, Bytes, BytesMut, Either, Ready};
+use ntex::util::{ready, BufMut, ByteString, Bytes, Either, PoolRef, Ready};
 use ntex_amqp_codec::protocol::{
     self as codec, Attach, DeliveryNumber, DeliveryState, Disposition, Error, Flow, MessageFormat,
     ReceiverSettleMode, Role, SenderSettleMode, SequenceNo, Target, TerminusDurability,
     TerminusExpiryPolicy, TransferBody,
 };
-use ntex_amqp_codec::Encode;
 
-use crate::cell::Cell;
-use crate::error::AmqpProtocolError;
 use crate::session::{Session, SessionInner, TransferState};
-use crate::Handle;
+use crate::{cell::Cell, codec::Encode, error::AmqpProtocolError, Handle};
 
 #[derive(Clone)]
 pub struct SenderLink {
@@ -34,6 +31,7 @@ pub(crate) struct SenderLinkInner {
     on_close: condition::Condition,
     on_disposition: Box<dyn Fn(Bytes, Result<Disposition, AmqpProtocolError>)>,
     max_message_size: Option<usize>,
+    pool: PoolRef,
 }
 
 struct PendingTransfer {
@@ -64,26 +62,32 @@ impl SenderLink {
         SenderLink { inner }
     }
 
+    /// Id of the sender link
     pub fn id(&self) -> u32 {
         self.inner.id as u32
     }
 
+    /// Name of the sender link
     pub fn name(&self) -> &ByteString {
         &self.inner.name
     }
 
+    /// Remote handle
     pub fn remote_handle(&self) -> Handle {
         self.inner.remote_handle
     }
 
+    /// Reference to session
     pub fn session(&self) -> &Session {
         &self.inner.get_ref().session
     }
 
-    pub fn session_mut(&mut self) -> &mut Session {
-        &mut self.inner.get_mut().session
+    /// Returns available send credit
+    pub fn credit(&self) -> u32 {
+        self.inner.get_ref().link_credit
     }
 
+    /// Send body
     pub fn send<T>(&self, body: T) -> impl Future<Output = Result<Disposition, AmqpProtocolError>>
     where
         T: Into<TransferBody>,
@@ -91,6 +95,7 @@ impl SenderLink {
         self.inner.get_mut().send(body, None)
     }
 
+    /// Send body with specific delivery tag
     pub fn send_with_tag<T>(
         &self,
         body: T,
@@ -124,10 +129,12 @@ impl SenderLink {
         self.inner.get_mut().settle_message(id, state);
     }
 
+    /// Close sender link
     pub fn close(&self) -> impl Future<Output = Result<(), AmqpProtocolError>> {
         self.inner.get_mut().close(None)
     }
 
+    /// Close sender link with error
     pub fn close_with_error<E>(
         &self,
         error: E,
@@ -163,9 +170,11 @@ impl SenderLinkInner {
         session: Cell<SessionInner>,
         max_message_size: Option<usize>,
     ) -> SenderLinkInner {
+        let pool = session.get_ref().memory_pool();
         SenderLinkInner {
             id,
             name,
+            pool,
             delivery_count,
             max_message_size,
             session: Session::new(session),
@@ -187,12 +196,16 @@ impl SenderLinkInner {
                 name = Some(addr.clone());
             }
         }
+        let pool = session.get_ref().memory_pool();
         let delivery_count = frame.initial_delivery_count().unwrap_or(0);
+        let mut name = name.unwrap_or_default();
+        name.trimdown();
 
         SenderLinkInner {
+            pool,
+            name,
             delivery_count,
             id: 0,
-            name: name.unwrap_or_default(),
             session: Session::new(session),
             remote_handle: frame.handle(),
             link_credit: 0,
@@ -323,17 +336,17 @@ impl SenderLinkInner {
             let max_frame_size = if max_frame_size > 2048 {
                 max_frame_size - 2048
             } else if max_frame_size == 0 {
-                usize::MAX
+                u32::MAX
             } else {
                 max_frame_size
-            };
+            } as usize;
 
             // body is larger than allowed frame size, send body as a set of transfers
             if body.len() > max_frame_size {
                 let mut body = match body {
                     TransferBody::Data(data) => data,
                     TransferBody::Message(msg) => {
-                        let mut buf = BytesMut::with_capacity(msg.encoded_size());
+                        let mut buf = self.pool.buf_with_capacity(msg.encoded_size());
                         msg.encode(&mut buf);
                         buf.freeze()
                     }
@@ -390,17 +403,17 @@ impl SenderLinkInner {
             let max_frame_size = if max_frame_size > 2048 {
                 max_frame_size - 2048
             } else if max_frame_size == 0 {
-                usize::MAX
+                u32::MAX
             } else {
                 max_frame_size
-            };
+            } as usize;
 
             // body is larger than allowed frame size, send body as a set of transfers
             if body.len() > max_frame_size {
                 let mut body = match body {
                     TransferBody::Data(data) => data,
                     TransferBody::Message(msg) => {
-                        let mut buf = BytesMut::with_capacity(msg.encoded_size());
+                        let mut buf = self.pool.buf_with_capacity(msg.encoded_size());
                         msg.encode(&mut buf);
                         buf.freeze()
                     }
@@ -486,7 +499,7 @@ impl SenderLinkInner {
             let delivery_tag = self.delivery_tag;
             self.delivery_tag = delivery_tag.saturating_add(1);
 
-            let mut buf = BytesMut::new();
+            let mut buf = self.pool.buf_with_capacity(16);
             buf.put_u32(delivery_tag);
             buf.freeze()
         })
@@ -529,11 +542,13 @@ impl SenderLinkBuilder {
         SenderLinkBuilder { frame, session }
     }
 
+    /// Set max message size
     pub fn max_message_size(mut self, size: u64) -> Self {
         self.frame.0.max_message_size = Some(size);
         self
     }
 
+    /// Modify attach frame
     pub fn with_frame<F>(mut self, f: F) -> Self
     where
         F: FnOnce(&mut Attach),
@@ -542,6 +557,7 @@ impl SenderLinkBuilder {
         self
     }
 
+    /// Initiate attach sender process
     pub async fn attach(self) -> Result<SenderLink, AmqpProtocolError> {
         let result = self
             .session
@@ -555,11 +571,6 @@ impl SenderLinkBuilder {
             Err(_) => Err(AmqpProtocolError::Disconnected),
         }
     }
-
-    #[doc(hidden)]
-    pub async fn open(self) -> Result<SenderLink, AmqpProtocolError> {
-        self.attach().await
-    }
 }
 
 enum Delivery {
@@ -573,10 +584,9 @@ impl Future for Delivery {
 
     fn poll(mut self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
         if let Delivery::Pending(ref mut receiver) = *self {
-            return match Pin::new(receiver).poll(cx) {
-                Poll::Ready(Ok(r)) => Poll::Ready(r),
-                Poll::Pending => Poll::Pending,
-                Poll::Ready(Err(e)) => {
+            return match ready!(Pin::new(receiver).poll(cx)) {
+                Ok(r) => Poll::Ready(r),
+                Err(e) => {
                     trace!("delivery oneshot is gone: {:?}", e);
                     Poll::Ready(Err(AmqpProtocolError::Disconnected))
                 }
