@@ -1,16 +1,19 @@
-use std::fmt;
+use std::{fmt, rc::Rc};
 
 use ntex::router::Path;
 use ntex::util::{ByteString, Either};
 
 use crate::codec::protocol::{Accepted, Attach, DeliveryState, Detach, Error, Flow, Rejected};
-use crate::{rcvlink::ReceiverLink, session::Session, sndlink::SenderLink, Handle, State};
+use crate::{
+    error::AmqpProtocolError, rcvlink::ReceiverLink, session::Session, sndlink::SenderLink, Handle,
+    State,
+};
 
 pub use crate::codec::protocol::Transfer;
 
 #[derive(Debug)]
 pub enum Message {
-    Attached(ReceiverLink),
+    Attached(Attach, ReceiverLink),
     // Detached(ReceiverLink),
     Transfer(ReceiverLink),
 }
@@ -18,25 +21,33 @@ pub enum Message {
 pub(crate) enum Action {
     None,
     AttachSender(SenderLink, Attach),
-    AttachReceiver(ReceiverLink),
+    AttachReceiver(ReceiverLink, Attach),
     DetachSender(SenderLink, Detach),
     DetachReceiver(ReceiverLink, Detach),
     SessionEnded(Vec<Either<SenderLink, ReceiverLink>>),
     Flow(SenderLink, Flow),
     Transfer(ReceiverLink),
+    RemoteClose(AmqpProtocolError),
 }
 
 pub struct Link<S> {
     pub(crate) state: State<S>,
     pub(crate) link: ReceiverLink,
     pub(crate) path: Path<ByteString>,
+    pub(crate) attach: Rc<Attach>,
 }
 
 impl<S> Link<S> {
-    pub(crate) fn new(link: ReceiverLink, state: State<S>, path: ByteString) -> Self {
+    pub(crate) fn new(
+        attach: Attach,
+        link: ReceiverLink,
+        state: State<S>,
+        path: ByteString,
+    ) -> Self {
         Link {
             state,
             link,
+            attach: Rc::new(attach),
             path: Path::new(path),
         }
     }
@@ -49,10 +60,6 @@ impl<S> Link<S> {
         &mut self.path
     }
 
-    pub fn frame(&self) -> &Attach {
-        self.link.frame()
-    }
-
     pub fn state(&self) -> &State<S> {
         &self.state
     }
@@ -61,12 +68,12 @@ impl<S> Link<S> {
         self.link.handle()
     }
 
-    pub fn session(&self) -> &Session {
-        self.link.session()
+    pub fn frame(&self) -> &Attach {
+        self.attach.as_ref()
     }
 
-    pub fn session_mut(&mut self) -> &mut Session {
-        self.link.session_mut()
+    pub fn session(&self) -> &Session {
+        self.link.session()
     }
 
     pub fn receiver(&self) -> &ReceiverLink {
@@ -88,6 +95,7 @@ impl<S> Clone for Link<S> {
             state: self.state.clone(),
             link: self.link.clone(),
             path: self.path.clone(),
+            attach: self.attach.clone(),
         }
     }
 }
@@ -95,7 +103,7 @@ impl<S> Clone for Link<S> {
 impl<S> fmt::Debug for Link<S> {
     fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
         fmt.debug_struct("Link<S>")
-            .field("frame", self.link.frame())
+            .field("frame", self.frame())
             .finish()
     }
 }
